package com.javacodegeeks.examples;

import java.security.KeyStore;
import java.security.SecureRandom;

import javax.net.ssl.SSLContext;
import javax.net.ssl.TrustManagerFactory;
import javax.xml.ws.BindingProvider;

import org.apache.cxf.configuration.jsse.TLSClientParameters;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import org.apache.cxf.transport.http.HTTPConduit;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;

@SpringBootApplication
public class SoapwebclientApplication implements CommandLineRunner {

	public static void main(String[] args) {
		SpringApplication.run(SoapwebclientApplication.class, args);
	}

	@Override
	public void run(String... args) throws Exception {
		RandomQuoteService service = new RandomQuoteService();
		RandomQuote port = service.getRandomQuotePort();
		
		// Override the endpoint address
		BindingProvider provider = (BindingProvider)port;
		provider.getRequestContext().put(
		     BindingProvider.ENDPOINT_ADDRESS_PROPERTY,
		     "https://localhost:9443/soapservice/services/quoteService");
		
		// Get the underlying http conduit of the client proxy
		Client client = ClientProxy.getClient(port);
	    HTTPConduit http = (HTTPConduit) client.getConduit();	
	    
	    // Set the TLS client parameters
	    TLSClientParameters parameters = new TLSClientParameters();
		parameters.setSSLSocketFactory(createSSLContext().getSocketFactory());
	    http.setTlsClientParameters(parameters);

		System.out.println("Quote of the day: " + port.getQuote());
		
	}

	private SSLContext createSSLContext() throws Exception {
		
		// Create and load the truststore
	    KeyStore trustStore = KeyStore.getInstance("JKS");
	    trustStore.load(SoapwebclientApplication.class.getClassLoader().getResourceAsStream("jks/client-truststore.jks"), "client-truststore-pass".toCharArray());

	    // Create and initialize the truststore manager
	    TrustManagerFactory tmf = TrustManagerFactory.getInstance("SunX509");
	    tmf.init(trustStore);

	    // Create and initialize the SSL context
	    SSLContext sslContext = SSLContext.getInstance("SSL");
	    sslContext.init(null, tmf.getTrustManagers(), new SecureRandom());
	    return sslContext;
	}	
}
